/* ##################################################################################################

Hacking-about of the ydrol hashtable posted on the MQL4 forum at https://www.mql5.com/en/code/11117,
and also available at http://lordy.co.nf/mql4/

This version is modified to use class templates which became available after the original 
ydrol code, in build 900+ of MT4. Comments largely removed from the code below (because 
they were falling out of step with the modifications). For an explanation of how the hashtable
actually works - which is unchanged; all the clever stuff is in the original - see the 
code at https://www.mql5.com/en/code/11117 and http://lordy.co.nf/mql4/

Original copyright notices (not clear what rights they're asserting): 
Copyright (C) 2014, Andrew Lord (NICKNAME=lordy) <forex@NICKNAME.org.uk> 
Copyright (C) 2002, 2004 Christopher Clark <firstname.lastname@cl.cam.ac.uk> 

The changes are mostly designed for elegance and simplicity, but they do address
one actual problem in the original. That goes as follows: in the original, all classes
stored in the hashtable must inherit from HashValue. MT4 does not allow
multiple inheritance. Therefore, you cannot put a derived class into the 
original hashtable because it cannot inherit from both its current base class
and also from HashValue.

This version now implements three hashtable classes: one for built-in basic data types
such as int, string, datetime etc; one for structs, such as MqlRates; and one for classes. 
(It's not possible in MQL4 at the moment to have a single, templated version which does 
all these things.)

The hashtables are created on a templated basis, specifying the type of key - which
can now be any basic datatype; no longer forced to be a string - and the type of value.

For example, to store a list of strings which are indexed by int keys:

   BasicHashTable<int, string> bht;
   bht.Set(23, "Text corresponding to item 23");
   ...
   string val = bht.Get(23);
   
Or, in the object version, to hold a list of MyClass objects which
are indexed by strings:

   ObjectHashTable<string, MyClass> oht; 
   oht.Set("Item1", objMyClass);
   ...
   MyClass * val = oht.Get("Item1");

Or, in the struct version, to hold a list of MqlRates structures which
are indexed by datetimes:

   StructHashTable<datetime, MqlRates> sht; 
   sht.Set(TimeCurrent(), mqlr);
   ...
   MqlRates val = sht.Get(TimeCurrent());
   
To recreate the original ydrol functionality where a single hashtable could
hold different types of object, simply have all your objects inherit
from a common base class (which was a requirement in the ydrol 
original), and then use ObjectHashTable<..., BaseClassName>
Alternatively, store the different types of object in the hashtable
using a wrapper class:

   class Wrapper {
   public:
      void * Value;
      Wrapper(void * X) {Value = X;}
      ~Wrapper() {delete Value;}
   };
   ...
   ObjectHashTable<string, Wrapper> oht;
   ...
   MyClass1 * obj1 = new MyClass1();
   MyClass2 * obj2 = new MyClass2();
   oht.Set("Item1", new Wrapper(obj1));
   oht.Set("Item2", new Wrapper(obj2));
   ...
   MyClass2 * val = (MyClass2*)oht.Get("Item2").Value;

Other changes from the ydrol original:

   * hPut, hGet, hDel, and hContainsKey have been renamed to:
     Set, Get, Remove, and ContainsKey

   * The helper methods such as hGetInt are no longer applicable
     and have been removed
     
   * Each of the classes has a ToArray() method, filling
     an array which is passed by reference and returning
     the number of items in the hashtable. For example:
         BasicHashTable<int, string> bht;
         bht.Set(1, "Item 1");
         bht.Set(2, "Item 2");
         bht.Set(3, "Item 3");
         string arr[];
         Print("Items in array should be 3... " , bht.ToArray(arr));

   * In the object version, you can no longer store
     null values. Set(key, NULL) now has the same 
     effect as Remove(key)

Other general notes arising from the additions:

   * The struct version will generate a compiler warning 
     about "possible use of uninitialized variable".
     To suppress this, see the two instances below 
     marked "NOTE A", and make the code change described there.     

   * You can't use StructHashTable with a structure
     which does not have a parameterless constructor.
     For example, the following will not work:
         struct MyStruct {
            int value;
            MyStruct(int x) {value = x;}
         };
      The hashtable doesn't and can't know how to 
      create such a struct. It doesn't know what value
      to supply to the compulsory constructor.
      You have to define this sort of thing as a class instead 
      of a struct.

   * What the struct version stores is a copy of the data
     passed into it. Therefore:
         StructHashTable<int, MqlRates> sht;
         MqlRates r;
         r.high = 1234;
         sht.Set(1, r);
         r.high = 1235;
         Print(sht.Get(1).high);
     This prints the value 1234, not 1235. 

   * If you Get() a non-existent key in a StructHashTable,
     what you get back is an unitialised structure of the
     relevant type.

   * If you Get() a non-existent key in a BasicHashTable,
     then what you get back is zero or an empty string.
     To distinguish between this versus actually-stored 
     zero/empty values, you need to use ContainsKey().
   
   * The object version retains the original functionality
     where, by default, it takes over ownership of the 
     objects, and deletes them when the hashtable is destroyed
     This ownership isn't applicable to the basic and struct
     versions. (As a result, those classes have different optional 
     constructors.)

      
################################################################################################## */


template <typename TKey, typename TValue>
class ObjectHashEntry {
    public:
        TKey _key;
        TValue * _val;
        ObjectHashEntry *_next;

        ObjectHashEntry() {
            _key=NULL;
            _val=NULL;
            _next=NULL;
        }

        ObjectHashEntry(TKey key,TValue * val) {
            _key=key;
            _val=val;
            _next=NULL;
        }

        ~ObjectHashEntry() {
        }
};


template <typename TKey, typename TValue>
class ObjectHashTable {

private:
    uint _hashSlots; 
    int _resizeThreshold;
    int _hashEntryCount;
    ObjectHashEntry<TKey, TValue> * _buckets[];
    bool _adoptValues;

    int _errCode;
    string _errText;

    void init(uint size,bool adoptValues)
    {
      uint temp[] = {
         17, 53, 97, 193, 389,
         769, 1543, 3079, 6151,
         12289, 24593, 49157, 98317,
         196613, 393241, 786433, 1572869,
         3145739, 6291469, 12582917, 25165843,
         50331653, 100663319, 201326611, 402653189,
         805306457, 1610612741};
         ArrayCopy(_primes, temp);
    
         _hashSlots = 0;
         _hashEntryCount = 0;
        clearError();
        setAdoptValues(adoptValues);

        rehash(size);
    }

    uint size2prime(uint size) {
        int pmax=ArraySize(_primes);
        for(int p=0 ; p<pmax; p++ ) {
            if (_primes[p] >= size) {
               return _primes[p];
            }
        }
        return size; 
    }

    uint _primes[];

    uint hash(TKey X)
    {
        string strType = typename(X);
        if (strType == "string") {        
            string s = (string)X;
            uchar c[];
            uint h = 0;
            
            if (s != NULL) {
               h = 5381;
               int n = StringToCharArray(s,c);
               for(int i = 0 ; i < n ; i++ ) {
                   h = ((h << 5 ) + h ) + c[i];
               }
            }
            return h % _hashSlots;
         } else {
            return (int)X % _hashSlots;
         }
    }
    void clearError() {
        setError(0,"");
    }
    void setError(int e,string m) {
        _errCode = e;
        _errText = m;
    }

public:

    ObjectHashTable() {
        init(17,true);
    }

    ObjectHashTable(bool adoptValues) {
        init(17,adoptValues);
    }

    ObjectHashTable(int size,bool adoptValues) {
        init(size,adoptValues);
    }

    ~ObjectHashTable() {
        // Free entries.
        for(uint i = 0 ; i< _hashSlots ; i++) {
            ObjectHashEntry<TKey, TValue> *nextEntry = NULL;
            for(ObjectHashEntry<TKey, TValue> *entry = _buckets[i] ; entry!= NULL ; entry = nextEntry ) 
            {
                nextEntry = entry._next;

                if (_adoptValues && entry._val != NULL && CheckPointer(entry._val) == POINTER_DYNAMIC ) {
                    delete entry._val;
                }
                delete entry;
            }
            _buckets[i] = NULL;
        }
    }

    int getErrCode() {
        return _errCode;
    }

    string getErrText() {
        return _errText;
    }

    void setAdoptValues(bool v) {
        _adoptValues = v;
    }

    bool getAdoptValues() {
        return _adoptValues;
    }

    private:
    uint _foundIndex;       // After find() is called is set to hashindex for name whether found or not.
    ObjectHashEntry<TKey, TValue> * _foundEntry; // After find() is called  is set to the HashEntry that contains the key.
    ObjectHashEntry<TKey, TValue>* _foundPrev;  // After find() is called  is set to the HashEntry before the entry
                            // (could use double linked list but requires more memory).

    bool find(TKey keyName) {
    
         //Alert("finding");
        bool found = false;

        // Get the index using the hashcode of the string
        _foundIndex = hash(keyName);
        

        if (_foundIndex>_hashSlots ) {

            setError(1,"hGet: bad hashIndex="+(string)_foundIndex+" size "+(string)_hashSlots);

        } else {

            // Search the linked list determined by the index.
            
            for(ObjectHashEntry<TKey, TValue> *e = _buckets[_foundIndex] ; e != NULL ; e = e._next )  {
                if (e._key == keyName) {
                    _foundEntry = e;
                    found=true;
                    break;
                }
                // Track the item before the target item in case deleting from single linked list.
                _foundPrev = e;
            }
        }

        return found;
    }

    public:

    /// This is used by the HashLoop class to get start of LinkedList at bucket[i]
    ObjectHashEntry<TKey, TValue> *getEntry(int i) {
        return _buckets[i];
    }

    /// Return the number of slots/buckets (not number of elements)
    uint getSlots() {
        return _hashSlots;
    }
    /// Return the number of elements in the Hash
    int getCount() {
        return _hashEntryCount;
    }

    /// Change the hash size and re-allocate values to new buckets.
    bool rehash(uint newSize) {
        bool ret = false;
        ObjectHashEntry<TKey, TValue>* oldTable[];

        uint oldSize = _hashSlots;
        newSize  = size2prime(newSize);
        //info("rehashing from "+(string)_hashSlots+" to "+(string)newSize+" "+(string)GetTickCount());

        if (newSize <= getSlots()) {
            setError(2,"rehash "+(string)newSize+" <= "+(string)_hashSlots);
        } else if (ArrayResize(_buckets,newSize) != newSize) {
            setError(3,"unable to resize ");
        } else if (ArrayResize(oldTable,oldSize) != oldSize) {
            setError(4,"unable to resize old copy ");
        } else {
            //Copy old table.
            for(uint i = 0 ; i < oldSize ; i++ ) oldTable[i] = _buckets[i];
            // Init new entries - not sure if MQL does this anyway
            for(uint icpy = 0 ; icpy<newSize ; icpy++ ) _buckets[icpy] = NULL;

            // Move entries to new slots
            _hashSlots = newSize;
            _resizeThreshold = (int)_hashSlots / 4 * 3; // Just use the default load factor value of Javas HashTable

            // Look through all slots
            for(uint oldHashCode = 0 ; oldHashCode<oldSize ; oldHashCode++ ) {
                ObjectHashEntry<TKey, TValue> *next = NULL;

                // Walk linked list
                for(ObjectHashEntry<TKey, TValue> *e = oldTable[oldHashCode] ; e != NULL ; e = next )  {

                    next = e._next;

                    uint newHashCode = hash(e._key);
                    // Insert at head of new list.
                    e._next = _buckets[newHashCode];
                    _buckets[newHashCode] = e;
                }

                oldTable[oldHashCode] = NULL;
            }
            ret = true;
        }
        return ret;
    }

    bool ContainsKey(TKey keyName) {
        return find(keyName);
    }

    TValue * Get(TKey  keyName) {

        TValue *obj = NULL;
        clearError();
        bool found=false;

        if (find(keyName)) {
            obj = _foundEntry._val;
        } else {
            //If Hash contains nulls then also check the errorCode=0 when retrieving
            if (!found) {
                setError(1,"not found");
            }
        }
        return obj;
    }

    TValue * Set(TKey keyName,TValue *obj) {
        if (obj == NULL) {
            Remove(keyName);
            return NULL;
        }    
        
        TValue *ret = NULL;
        clearError();
         
        if (find(keyName)) {
            // Return revious value
            ret = _foundEntry._val;

            // Replace entry contents
            if (_adoptValues && _foundEntry._val != NULL && CheckPointer(_foundEntry._val) == POINTER_DYNAMIC ) {
                delete _foundEntry._val;
            }
            _foundEntry._val = obj;

        } else {
            // Insert new entry at head of list
            ObjectHashEntry<TKey, TValue>* e = new ObjectHashEntry<TKey, TValue>(keyName,obj);
            ObjectHashEntry<TKey, TValue>* first = _buckets[_foundIndex];
            e._next = first;
            _buckets[_foundIndex] = e;
            _hashEntryCount++;

            //info((string)_hashEntryCount+" vs. "+(string)_resizeThreshold);
            // Auto Resize if number of entries hits _resizeThreshold
            if (_hashEntryCount > _resizeThreshold ) {
                rehash(_hashSlots/2*3); // this will snap to the next prime
            }
        }
        return ret;
    }
    /// Delete an entry from the hash.
    bool Remove(TKey keyName) {

        bool found = false;
        clearError();

        if (find(keyName)) {
            ObjectHashEntry<TKey, TValue> *next = _foundEntry._next;
            if (_foundPrev != NULL) {
                //Remove entry from the middle of the list.
                _foundPrev._next = next;
            } else {
                // remove from head of list
                _buckets[_foundIndex] = next;
            }

            if (_adoptValues && _foundEntry._val != NULL&& CheckPointer(_foundEntry._val) == POINTER_DYNAMIC) {
                delete _foundEntry._val;
            }
            delete _foundEntry;
            _hashEntryCount--;
            found=true;

        }
        return found;
    }

    int ToArray(TValue* & arr[]) {
         int ctAlloc = 0, ctTotal = 0;
         ObjectHashLoop<TKey, TValue> *l;
         ArrayResize(arr, 0);
         
         for (l = new ObjectHashLoop<TKey, TValue>(GetPointer(this)) ; l.hasNext() ; l.next()  ) {
            ctTotal++;
            while (ctAlloc < ctTotal) {
               ctAlloc += 100;
               ArrayResize(arr, ctAlloc);
            }
            arr[ctTotal - 1] = l.val();
         }
         delete l;
         ArrayResize(arr, ctTotal);
         return ctTotal;
    }
};


template <typename TKey, typename TValue>
class ObjectHashLoop {
    private:
        uint _index;
        ObjectHashEntry<TKey, TValue> *_currentEntry;
        ObjectHashTable<TKey, TValue> *_hash;

    public:
        /// Create iterator for a hash - move to first item
        ObjectHashLoop(ObjectHashTable<TKey, TValue> *h) {
            setHash(h);
        }
        ~ObjectHashLoop() {};

        /// Clear current state and move to first item (if any).
        void reset() {
            _index=0;
            _currentEntry = _hash.getEntry(_index);

            // Move to first item
            if (_currentEntry == NULL) {
                next();
            }
        }

        /// Change the hash over which to iterate.
        void setHash(ObjectHashTable<TKey, TValue> *h) {
            _hash = h;
            reset();
        }

        /// Check if more items.
        bool hasNext() {
            bool ret = ( _currentEntry != NULL);
            //config("hasNext=",ret);
            return ret;
        }

        /// Move to next item.
        void next() {

            //config("next : index = ",_index);

            // Advance
            if (_currentEntry != NULL) {
                _currentEntry = _currentEntry._next;
            }

            // Keep advancing if _currentEntry is null
            while (_currentEntry==NULL) {
                _index++;
                if (_index >= _hash.getSlots() ) return ;
                _currentEntry = _hash.getEntry(_index);
            }
        }

        /// Return the key name of the current item.
        TKey  key() {
            if (_currentEntry != NULL) {
                return _currentEntry._key;
            } else {
                return NULL;
            }
        }

        /// Return the value.
        TValue *val() {
            if (_currentEntry != NULL) {
                return _currentEntry._val;
            } else {
                return NULL;
            }
        }
};


// ----------------------------------------------------------------------------------------------------------
template <typename TKey, typename TValue>
class BasicHashEntry {
    public:
        TKey _key;
        TValue _val;
        BasicHashEntry *_next;

        BasicHashEntry() {
            _key=NULL;
            _val=NULL;
            _next=NULL;
        }

        BasicHashEntry(TKey key, TValue val) {
            _key=key;
            _val=val;
            _next=NULL;
        }

        ~BasicHashEntry() {
        }
};


template <typename TKey, typename TValue>
class BasicHashTable {

private:
    uint _hashSlots; 
    int _resizeThreshold;
    int _hashEntryCount;
    BasicHashEntry<TKey, TValue> * _buckets[];

    int _errCode;
    string _errText;

    void init(uint size,bool adoptValues)
    {
      uint temp[] = {
         17, 53, 97, 193, 389,
         769, 1543, 3079, 6151,
         12289, 24593, 49157, 98317,
         196613, 393241, 786433, 1572869,
         3145739, 6291469, 12582917, 25165843,
         50331653, 100663319, 201326611, 402653189,
         805306457, 1610612741};
         ArrayCopy(_primes, temp);
    
         _hashSlots = 0;
         _hashEntryCount = 0;
        clearError();

        rehash(size);
    }

    // Hash table distribution is better when size is prime, eg if hash function procduces numbers
    // that are multiples of x, then there may be grouping occuring around gcd(x,slots) gcd(2x,slots) etc
    // using a prime size helps spread the distribution.
    uint size2prime(uint size) {
        int pmax=ArraySize(_primes);
        for(int p=0 ; p<pmax; p++ ) {
            if (_primes[p] >= size) {
               return _primes[p];
            }
        }
        return size; 
    }

    /// Primes that approx double in size, used for hash table sizes to avoid gcd causing bunching
    uint _primes[];

    uint hash(TKey X)
    {
        string strType = typename(X);
        if (strType == "string") {        
            string s = (string)X;
            uchar c[];
            uint h = 0;
            
            if (s != NULL) {
               h = 5381;
               int n = StringToCharArray(s,c);
               for(int i = 0 ; i < n ; i++ ) {
                   h = ((h << 5 ) + h ) + c[i];
               }
            }
            return h % _hashSlots;
         } else {
            return (int)X % _hashSlots;
         }
    }
    void clearError() {
        setError(0,"");
    }
    void setError(int e,string m) {
        _errCode = e;
        _errText = m;
        //error((string)e,m);
    }

public:

    BasicHashTable() {
        init(17,true);
    }


    BasicHashTable(bool adoptValues) {
        init(17,adoptValues);
    }

    BasicHashTable(int size,bool adoptValues) {
        init(size,adoptValues);
    }

    ~BasicHashTable() {
        // Free entries.
        for(uint i = 0 ; i< _hashSlots ; i++) {
            BasicHashEntry<TKey, TValue> *nextEntry = NULL;
            for(BasicHashEntry<TKey, TValue> *entry = _buckets[i] ; entry!= NULL ; entry = nextEntry ) 
            {
                nextEntry = entry._next;

                delete entry;
            }
            _buckets[i] = NULL;
        }
    }

    int getErrCode() {
        return _errCode;
    }
    /// Return text of the error message.
    string getErrText() {
        return _errText;
    }

    private:
    uint _foundIndex;       // After find() is called is set to hashindex for name whether found or not.
    BasicHashEntry<TKey, TValue> * _foundEntry; // After find() is called  is set to the HashEntry that contains the key.
    BasicHashEntry<TKey, TValue>* _foundPrev;  // After find() is called  is set to the HashEntry before the entry
                            // (could use double linked list but requires more memory).

    /// Look for the required entry for key 'name' true if found.
    bool find(TKey keyName) {
    
         //Alert("finding");
        bool found = false;

        // Get the index using the hashcode of the string
        _foundIndex = hash(keyName);
        

        if (_foundIndex>_hashSlots ) {

            setError(1,"hGet: bad hashIndex="+(string)_foundIndex+" size "+(string)_hashSlots);

        } else {

            // Search the linked list determined by the index.
            
            for(BasicHashEntry<TKey, TValue> *e = _buckets[_foundIndex] ; e != NULL ; e = e._next )  {
                if (e._key == keyName) {
                    _foundEntry = e;
                    found=true;
                    break;
                }
                // Track the item before the target item in case deleting from single linked list.
                _foundPrev = e;
            }
        }

        return found;
    }

    public:

    /// This is used by the HashLoop class to get start of LinkedList at bucket[i]
    BasicHashEntry<TKey, TValue> *getEntry(int i) {
        return _buckets[i];
    }

    /// Return the number of slots/buckets (not number of elements)
    uint getSlots() {
        return _hashSlots;
    }
    /// Return the number of elements in the Hash
    int getCount() {
        return _hashEntryCount;
    }

    /// Change the hash size and re-allocate values to new buckets.
    bool rehash(uint newSize) {
        bool ret = false;
        BasicHashEntry<TKey, TValue>* oldTable[];

        uint oldSize = _hashSlots;
        newSize  = size2prime(newSize);
        //info("rehashing from "+(string)_hashSlots+" to "+(string)newSize+" "+(string)GetTickCount());

        if (newSize <= getSlots()) {
            setError(2,"rehash "+(string)newSize+" <= "+(string)_hashSlots);
        } else if (ArrayResize(_buckets,newSize) != newSize) {
            setError(3,"unable to resize ");
        } else if (ArrayResize(oldTable,oldSize) != oldSize) {
            setError(4,"unable to resize old copy ");
        } else {
            //Copy old table.
            for(uint i = 0 ; i < oldSize ; i++ ) oldTable[i] = _buckets[i];
            // Init new entries - not sure if MQL does this anyway
            for(uint icpy = 0 ; icpy<newSize ; icpy++ ) _buckets[icpy] = NULL;

            // Move entries to new slots
            _hashSlots = newSize;
            _resizeThreshold = (int)_hashSlots / 4 * 3; // Just use the default load factor value of Javas HashTable

            // Look through all slots
            for(uint oldHashCode = 0 ; oldHashCode<oldSize ; oldHashCode++ ) {
                BasicHashEntry<TKey, TValue> *next = NULL;

                // Walk linked list
                for(BasicHashEntry<TKey, TValue> *e = oldTable[oldHashCode] ; e != NULL ; e = next )  {

                    next = e._next;

                    uint newHashCode = hash(e._key);
                    // Insert at head of new list.
                    e._next = _buckets[newHashCode];
                    _buckets[newHashCode] = e;
                }

                oldTable[oldHashCode] = NULL;
            }
            ret = true;
        }
        return ret;
    }

    bool ContainsKey(TKey keyName) {
        return find(keyName);
    }

    TValue Get(TKey  keyName) {

        TValue obj = NULL;
        clearError();
        bool found=false;

        if (find(keyName)) {
            obj = _foundEntry._val;
        } else {
            //If Hash contains nulls then also check the errorCode=0 when retrieving
            if (!found) {
                setError(1,"not found");
            }
        }
        return obj;
    }

    TValue Set(TKey keyName, TValue obj) {
    
        TValue ret = NULL;
        clearError();
         
        if (find(keyName)) {
            // Return revious value
            ret = _foundEntry._val;
            /*
            // Replace entry contents
            if (_adoptValues && _foundEntry._val != NULL && CheckPointer(_foundEntry._val) == POINTER_DYNAMIC ) {
                delete _foundEntry._val;
            }
            */
            _foundEntry._val = obj;

        } else {
            // Insert new entry at head of list
            BasicHashEntry<TKey, TValue>* e = new BasicHashEntry<TKey, TValue>(keyName,obj);
            BasicHashEntry<TKey, TValue>* first = _buckets[_foundIndex];
            e._next = first;
            _buckets[_foundIndex] = e;
            _hashEntryCount++;

            //info((string)_hashEntryCount+" vs. "+(string)_resizeThreshold);
            // Auto Resize if number of entries hits _resizeThreshold
            if (_hashEntryCount > _resizeThreshold ) {
                rehash(_hashSlots/2*3); // this will snap to the next prime
            }
        }
        return ret;
    }
    /// Delete an entry from the hash.
    bool Remove(TKey keyName) {

        bool found = false;
        clearError();

        if (find(keyName)) {
            BasicHashEntry<TKey, TValue> *next = _foundEntry._next;
            if (_foundPrev != NULL) {
                //Remove entry from the middle of the list.
                _foundPrev._next = next;
            } else {
                // remove from head of list
                _buckets[_foundIndex] = next;
            }

            delete _foundEntry;
            _hashEntryCount--;
            found=true;

        }
        return found;
    }
    
    int ToArray(TValue & arr[]) {
         int ctAlloc = 0, ctTotal = 0;
         BasicHashLoop<TKey, TValue> *l;
         ArrayResize(arr, 0);
         
         for (l = new BasicHashLoop<TKey, TValue>(GetPointer(this)) ; l.hasNext() ; l.next()  ) {
            ctTotal++;
            while (ctAlloc < ctTotal) {
               ctAlloc += 100;
               ArrayResize(arr, ctAlloc);
            }
            arr[ctTotal - 1] = l.val();
         }
         delete l;
         ArrayResize(arr, ctTotal);
         return ctTotal;
    }
};


template <typename TKey, typename TValue>
class BasicHashLoop {
    private:
        uint _index;
        BasicHashEntry<TKey, TValue> *_currentEntry;
        BasicHashTable<TKey, TValue> *_hash;

    public:
        /// Create iterator for a hash - move to first item
        BasicHashLoop(BasicHashTable<TKey, TValue> *h) {
            setHash(h);
        }
        ~BasicHashLoop() {};

        /// Clear current state and move to first item (if any).
        void reset() {
            _index=0;
            _currentEntry = _hash.getEntry(_index);

            // Move to first item
            if (_currentEntry == NULL) {
                next();
            }
        }

        /// Change the hash over which to iterate.
        void setHash(BasicHashTable<TKey, TValue> *h) {
            _hash = h;
            reset();
        }

        /// Check if more items.
        bool hasNext() {
            bool ret = ( _currentEntry != NULL);
            //config("hasNext=",ret);
            return ret;
        }

        /// Move to next item.
        void next() {

            //config("next : index = ",_index);

            // Advance
            if (_currentEntry != NULL) {
                _currentEntry = _currentEntry._next;
            }

            // Keep advancing if _currentEntry is null
            while (_currentEntry==NULL) {
                _index++;
                if (_index >= _hash.getSlots() ) return ;
                _currentEntry = _hash.getEntry(_index);
            }
        }

        /// Return the key name of the current item.
        TKey  key() {
            if (_currentEntry != NULL) {
                return _currentEntry._key;
            } else {
                return NULL;
            }
        }

        /// Return the value.
        TValue val() {
            if (_currentEntry != NULL) {
                return _currentEntry._val;
            } else {
                return NULL;
            }
        }
};


// ----------------------------------------------------------------------------------------------------------
template <typename TKey, typename TValue>
class StructHashEntry {
    public:
        TKey _key;
        TValue _val;
        StructHashEntry * _next;

        StructHashEntry() {
            _key=NULL;
            _next=NULL;
        }

        StructHashEntry(TKey key, TValue & val) {
            _key=key;
            _val=val;
            _next=NULL;
        }

        ~StructHashEntry() {
        }
};


template <typename TKey, typename TValue>
class StructHashTable {

private:
    uint _hashSlots; 
    int _resizeThreshold;
    int _hashEntryCount;
    StructHashEntry<TKey, TValue> * _buckets[];

    int _errCode;
    string _errText;

    void init(uint size,bool adoptValues)
    {
      uint temp[] = {
         17, 53, 97, 193, 389,
         769, 1543, 3079, 6151,
         12289, 24593, 49157, 98317,
         196613, 393241, 786433, 1572869,
         3145739, 6291469, 12582917, 25165843,
         50331653, 100663319, 201326611, 402653189,
         805306457, 1610612741};
         ArrayCopy(_primes, temp);
    
         _hashSlots = 0;
         _hashEntryCount = 0;
        clearError();

        rehash(size);
    }

    // Hash table distribution is better when size is prime, eg if hash function procduces numbers
    // that are multiples of x, then there may be grouping occuring around gcd(x,slots) gcd(2x,slots) etc
    // using a prime size helps spread the distribution.
    uint size2prime(uint size) {
        int pmax=ArraySize(_primes);
        for(int p=0 ; p<pmax; p++ ) {
            if (_primes[p] >= size) {
               return _primes[p];
            }
        }
        return size; 
    }

    /// Primes that approx double in size, used for hash table sizes to avoid gcd causing bunching
    uint _primes[];

    uint hash(TKey X)
    {
        string strType = typename(X);
        if (strType == "string") {        
            string s = (string)X;
            uchar c[];
            uint h = 0;
            
            if (s != NULL) {
               h = 5381;
               int n = StringToCharArray(s,c);
               for(int i = 0 ; i < n ; i++ ) {
                   h = ((h << 5 ) + h ) + c[i];
               }
            }
            return h % _hashSlots;
         } else {
            return (int)X % _hashSlots;
         }
    }
    void clearError() {
        setError(0,"");
    }
    void setError(int e,string m) {
        _errCode = e;
        _errText = m;
        //error((string)e,m);
    }

public:

    StructHashTable() {
        init(17,true);
    }


    StructHashTable(bool adoptValues) {
        init(17,adoptValues);
    }

    StructHashTable(int size,bool adoptValues) {
        init(size,adoptValues);
    }

    ~StructHashTable() {
        // Free entries.
        for(uint i = 0 ; i< _hashSlots ; i++) {
            StructHashEntry<TKey, TValue> *nextEntry = NULL;
            for(StructHashEntry<TKey, TValue> *entry = _buckets[i] ; entry!= NULL ; entry = nextEntry ) 
            {
                nextEntry = entry._next;

                delete entry;
            }
            _buckets[i] = NULL;
        }
    }

    int getErrCode() {
        return _errCode;
    }
    /// Return text of the error message.
    string getErrText() {
        return _errText;
    }

    private:
    uint _foundIndex;       // After find() is called is set to hashindex for name whether found or not.
    StructHashEntry<TKey, TValue> * _foundEntry; // After find() is called  is set to the HashEntry that contains the key.
    StructHashEntry<TKey, TValue>* _foundPrev;  // After find() is called  is set to the HashEntry before the entry
                            // (could use double linked list but requires more memory).

    /// Look for the required entry for key 'name' true if found.
    bool find(TKey keyName) {
    
         //Alert("finding");
        bool found = false;

        // Get the index using the hashcode of the string
        _foundIndex = hash(keyName);

        if (_foundIndex>_hashSlots ) {
            setError(1,"hGet: bad hashIndex="+(string)_foundIndex+" size "+(string)_hashSlots);

        } else {

            // Search the linked list determined by the index.
            
            for(StructHashEntry<TKey, TValue> *e = _buckets[_foundIndex] ; e != NULL ; e = e._next )  {
                if (e._key == keyName) {
                    _foundEntry = e;
                    found=true;
                    break;
                }
                // Track the item before the target item in case deleting from single linked list.
                _foundPrev = e;
            }
        }

        return found;
    }

    public:

    /// This is used by the HashLoop class to get start of LinkedList at bucket[i]
    StructHashEntry<TKey, TValue> *getEntry(int i) {
        return _buckets[i];
    }

    /// Return the number of slots/buckets (not number of elements)
    uint getSlots() {
        return _hashSlots;
    }
    /// Return the number of elements in the Hash
    int getCount() {
        return _hashEntryCount;
    }

    /// Change the hash size and re-allocate values to new buckets.
    bool rehash(uint newSize) {
        bool ret = false;
        StructHashEntry<TKey, TValue>* oldTable[];

        uint oldSize = _hashSlots;
        newSize  = size2prime(newSize);
        //info("rehashing from "+(string)_hashSlots+" to "+(string)newSize+" "+(string)GetTickCount());

        if (newSize <= getSlots()) {
            setError(2,"rehash "+(string)newSize+" <= "+(string)_hashSlots);
        } else if (ArrayResize(_buckets,newSize) != newSize) {
            setError(3,"unable to resize ");
        } else if (ArrayResize(oldTable,oldSize) != oldSize) {
            setError(4,"unable to resize old copy ");
        } else {
            //Copy old table.
            for(uint i = 0 ; i < oldSize ; i++ ) oldTable[i] = _buckets[i];
            // Init new entries - not sure if MQL does this anyway
            for(uint icpy = 0 ; icpy<newSize ; icpy++ ) _buckets[icpy] = NULL;

            // Move entries to new slots
            _hashSlots = newSize;
            _resizeThreshold = (int)_hashSlots / 4 * 3; // Just use the default load factor value of Javas HashTable

            // Look through all slots
            for(uint oldHashCode = 0 ; oldHashCode<oldSize ; oldHashCode++ ) {
                StructHashEntry<TKey, TValue> *next = NULL;

                // Walk linked list
                for(StructHashEntry<TKey, TValue> *e = oldTable[oldHashCode] ; e != NULL ; e = next )  {

                    next = e._next;

                    uint newHashCode = hash(e._key);
                    // Insert at head of new list.
                    e._next = _buckets[newHashCode];
                    _buckets[newHashCode] = e;
                }

                oldTable[oldHashCode] = NULL;
            }
            ret = true;
        }
        return ret;
    }

    bool ContainsKey(TKey keyName) {
        return find(keyName);
    }

    TValue Get(TKey keyName) {
        clearError();
        bool found=false;

        if (find(keyName)) {
            return _foundEntry._val;
        } else {
            //If Hash contains nulls then also check the errorCode=0 when retrieving
            if (!found) {
                setError(1,"not found");
            }
        }
        
        // [NOTE A]
        // This causes a "possible use of uninitialized variable".
        // The issue is that we can't do   TValue empty = {};   
        // because that causes a compiler error, rather than just a 
        // warning, on any structure which has an explicit constructor.
        // But if you are only going to use the code with structures
        // which don't have a constructor, then you can make that change.
        TValue empty;

        return empty;
    }

    void Set(TKey keyName, TValue & obj) {
        clearError();
         
        if (find(keyName)) {
            // Return revious value
            _foundEntry._val = obj;

        } else {
            // Insert new entry at head of list
            StructHashEntry<TKey, TValue>* e = new StructHashEntry<TKey, TValue>(keyName,obj);
            StructHashEntry<TKey, TValue>* first = _buckets[_foundIndex];
            e._next = first;
            _buckets[_foundIndex] = e;
            _hashEntryCount++;

            //info((string)_hashEntryCount+" vs. "+(string)_resizeThreshold);
            // Auto Resize if number of entries hits _resizeThreshold
            if (_hashEntryCount > _resizeThreshold ) {
                rehash(_hashSlots/2*3); // this will snap to the next prime
            }
        }
    }
    /// Delete an entry from the hash.
    bool Remove(TKey keyName) {

        bool found = false;
        clearError();

        if (find(keyName)) {
            StructHashEntry<TKey, TValue> *next = _foundEntry._next;
            if (_foundPrev != NULL) {
                //Remove entry from the middle of the list.
                _foundPrev._next = next;
            } else {
                // remove from head of list
                _buckets[_foundIndex] = next;
            }

            delete _foundEntry;
            _hashEntryCount--;
            found=true;

        }
        return found;
    }
    
    int ToArray(TValue & arr[]) {
         int ctAlloc = 0, ctTotal = 0;
         StructHashLoop<TKey, TValue> *l;
         ArrayResize(arr, 0);
         
         for (l = new StructHashLoop<TKey, TValue>(GetPointer(this)) ; l.hasNext() ; l.next()  ) {
            ctTotal++;
            while (ctAlloc < ctTotal) {
               ctAlloc += 100;
               ArrayResize(arr, ctAlloc);
            }
            arr[ctTotal - 1] = l.val();
         }
         delete l;
         ArrayResize(arr, ctTotal);
         return ctTotal;
    }
};


template <typename TKey, typename TValue>
class StructHashLoop {
    private:
        uint _index;
        StructHashEntry<TKey, TValue> *_currentEntry;
        StructHashTable<TKey, TValue> *_hash;

    public:
        /// Create iterator for a hash - move to first item
        StructHashLoop(StructHashTable<TKey, TValue> *h) {
            setHash(h);
        }
        ~StructHashLoop() {};

        /// Clear current state and move to first item (if any).
        void reset() {
            _index=0;
            _currentEntry = _hash.getEntry(_index);

            // Move to first item
            if (_currentEntry == NULL) {
                next();
            }
        }

        /// Change the hash over which to iterate.
        void setHash(StructHashTable<TKey, TValue> *h) {
            _hash = h;
            reset();
        }

        /// Check if more items.
        bool hasNext() {
            bool ret = ( _currentEntry != NULL);
            //config("hasNext=",ret);
            return ret;
        }

        /// Move to next item.
        void next() {

            //config("next : index = ",_index);

            // Advance
            if (_currentEntry != NULL) {
                _currentEntry = _currentEntry._next;
            }

            // Keep advancing if _currentEntry is null
            while (_currentEntry==NULL) {
                _index++;
                if (_index >= _hash.getSlots() ) return ;
                _currentEntry = _hash.getEntry(_index);
            }
        }

        /// Return the key name of the current item.
        TKey  key() {
            if (_currentEntry != NULL) {
                return _currentEntry._key;
            } else {
                return NULL;
            }
        }

        /// Return the value.
        TValue val() {
            if (_currentEntry != NULL) {
                return _currentEntry._val;
            } else {
              // [NOTE A]
              // This causes a "possible use of uninitialized variable".
              // The issue is that we can't do   TValue empty = {};   
              // because that causes a compiler error, rather than just a 
              // warning, on any structure which has an explicit constructor.
              // But if you are only going to use the code with structures
              // which don't have a constructor, then you can make that change.
               TValue empty;

               return empty;
            }
        }
};
